import sys, os, subprocess

from subprocess import Popen, PIPE, STDOUT
from Tkinter import *
from tkFileDialog import *
from tkMessageBox import *
import tempfile
import shutil


root = Tk()

class logic:
    def __init__(self):
        
        #gets path of self and subsequently the romtool bin      
        self.path_to_self = os.getcwd()
        self.romtool = "\'" + self.path_to_self + "/rom_tool\'"
        
        self.rom_is_valid = False
        
        print os.getcwd()
        
    def choose_rom(self):

        self.rom_full_path = askopenfilename()
                
        if self.rom_full_path :
            path_sans_extension, extension = os.path.splitext(self.rom_full_path)
            path, rom_filename = os.path.split(self.rom_full_path)
            
            if extension == ".3ds" or extension == ".cci" :
                
                mainui.label_rom_name_text.set(rom_filename)
                
                self.rom_is_valid = True
                self.get_rom_info()

                
            else :
                output = "\n " + rom_filename + " is not a valid 3ds rom."
                
                mainui.label_rom_name_text.set("Invalid Filetype (" + extension +")" )
                
                mainui.update_rom_info_text(output)

    
    
    def get_rom_info(self):
        if self.rom_is_valid == False:
            pass
        else:    
            command = self.romtool + " -i \'" + self.rom_full_path + "\'"        
            self.run_command(command)
    
    def trim_rom(self):
        if self.rom_is_valid == False:
            pass
        else:
            command = self.romtool + " -t \'" + self.rom_full_path + "\'"
            self.run_command(command)

    def un_trim_rom(self):
        if self.rom_is_valid == False:
            pass
        else:
            command = self.romtool + " -r \'" + self.rom_full_path + "\'"
            self.run_command(command)
    
    def get_rom_partition_info(self):
        if self.rom_is_valid == False:
            pass
        else:
            command = self.romtool + " -p \'" + self.rom_full_path + "\'"
            self.run_command(command)        
    
    def unpack_rom(self):
        if self.rom_is_valid == False:
            pass
        else:
            # a kind reminder to wait
            self.response = "\n Please wait..."
            mainui.update_rom_info_text(self.response)
            
            unpack_directory = askdirectory(title = "Output Folder")
            
            if unpack_directory:
                
                # unpack to tmp folder
                # must be done because rom_tool doesn't like certain paths and instead extracts to its own folder
                
                temp_folder = tempfile.mkdtemp()

                command = self.romtool + " -x \'" + temp_folder + "\' \'" + self.rom_full_path +"\'"
                
                self.run_command(command)
                
                # get list of unpacked files
                source = os.listdir(temp_folder)

                for files in source:
                    files = temp_folder + "/" + files # create full path
                    shutil.move(files, unpack_directory ) # move to chosen folder
                
                os.rmdir(temp_folder) # delete temp folder
                
                
            else:
                # if the user cancelled, get rom info to replace wait text
                command = self.romtool + " -i \'" + self.rom_full_path + "\'"        
                self.run_command(command)
            
    def trim_update(self):
        if self.rom_is_valid == False:
            pass
        else:
            command = self.romtool + " -u \'" + self.rom_full_path + "\'"
            # must trim rom after update trim to avoid err -6
            command_part_2 = self.romtool + " -t \'" + self.rom_full_path + "\'"
            
            
            message = " Trimming update data cannot be undone. \n Continue?"
            
            if askyesno(title = "Warning", message = message):
                self.run_command(command)
                self.run_command(command_part_2)
            else:
                pass
            

    def run_command(self, command):        
        try:
            output = subprocess.check_output(command, shell=True, stderr=subprocess.STDOUT)
        except subprocess.CalledProcessError as err:
            output = err

        mainui.update_rom_info_text(output)

class MainUI(Frame):
    
    def __init__(self, parent):
        Frame.__init__(self, parent, background="white")   
         
        self.parent = parent
        
        self.parent.title("Romtool GUI")

        self.create_window()    #Learned the hard way, these must go in order!
        self.create_buttons()
        self.create_text_widgets()
        #self.create_menus()        
        self.center_window()
        
        root.columnconfigure(2,minsize=50)
        
        # fill text widgets
        
        self.update_rom_info_text("\n Please Choose a Rom.")
        
    def create_window(self):
        self.parent.resizable(0,0) # stops user from resizing window
                             
    def center_window(self): # Centers window when opened
        # Apparently a common hack to get the window size. Temporarily hide the
        # window to avoid update_idletasks() drawing the window in the wrong
        # position.
        root.withdraw()
        root.update_idletasks()  # Update "requested size" from geometry manager
        
        
        x = (root.winfo_screenwidth() - root.winfo_reqwidth()) / 2
        y = (root.winfo_screenheight() - root.winfo_reqheight()) / 2
        root.geometry("+%d+%d" % (x, y)) # math and stuff
        
        root.deiconify() # redraws window

    def create_buttons(self):
        Button(root, text='Choose Rom', takefocus = 1, command=logic.choose_rom).grid(row = 0, column = 0)
        
        Button(root, text='Trim', width = 14, command=logic.trim_rom).grid(row=2, column = 0)
        Button(root, text="Un-Trim", width = 14, command = logic.un_trim_rom).grid(row = 2, column = 1)
        Button(root, text="Remove Update", width = 14, command = logic.trim_update).grid(row = 2, column = 2)
        Button(root, text="Unpack", width = 14, command = logic.unpack_rom).grid(row = 3, column = 0)
        Button(root, text="Partition Info", width = 14, command = logic.get_rom_partition_info).grid(row = 3, column = 1)
        Button(root, text="Rom Info", width = 14, command = logic.get_rom_info).grid(row = 3, column = 2)
        
    def create_text_widgets(self):
        
        # Icon
        pic = logic.path_to_self + "/logo.gif"
        
        # must be assigned to self or gets garbage collected.
        self.logo = PhotoImage(file=pic)

        self.label_icon = Label(root, image=self.logo).grid(row = 0, column = 2)
        
        
        
        # Rom Name
        self.label_rom_name_text = StringVar()
        Label(root, textvariable=self.label_rom_name_text, wraplength = 170).grid(row = 0, column = 1, sticky = W)
        
        # Rom Info

        self.text_rom_info_text = Text(root)
        self.text_rom_info_text.grid(row = 1, column = 0, columnspan = 3, sticky = W, padx = 4)
        self.text_rom_info_text.config(background='#dddddd')
        
        
        
    def update_rom_info_text(self, response):
        
        self.text_rom_info_text.config(state='normal') # allow editing
        self.text_rom_info_text.delete("1.0", END) # clear
        self.text_rom_info_text.insert("1.0", response)  # update
        self.text_rom_info_text.config(state='disabled') # disable editing
        
        
if __name__ == '__main__':
    logic = logic()
    mainui = MainUI(root)    
    root.mainloop()